# -*- coding: utf-8 -*-
"""
/***************************************************************************
 CtrlCraftDialogChartEstimatedBlocs
                                 A QGIS plugin
 Areva - plugin for displaying 3D mines
                             -------------------
        begin                : 2017-09-12
        git sha              : $Format:%H$
        copyright            : (C) 2017 by Oslandia
        email                : sylvain.beorchia@oslandia.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

import os
import csv
import configparser

from PyQt5 import uic
from PyQt5.QtWidgets import (QFileDialog,
                         qApp,
                         QLabel,
                         QDialog)

from qgis.gui import QgsMessageBar

from CtrlCraft import CtrlCraftLogger

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'ui', 'import_db.ui'))


class CtrlCraftDatabaseExport(QDialog, FORM_CLASS):
    """
    Dialog to import Slabs in database.
    """

    def __init__(self, data, projIni, deposit, parent=None):
        super(CtrlCraftDatabaseExport, self).__init__(parent)
        self.setupUi(self)
        self.projIni = projIni
        self.deposit = deposit
        self.data = data
        self.config = configparser.ConfigParser()
        self.config.read(self.projIni)

        #self.__populateProjComboBox()
        self.mCancelButton.clicked.connect(self.__close)
        self.mImportButton.setText( self.tr('Export') )
        self.mImportButton.clicked.connect(self.__export)
        self.mSelectButton.clicked.connect(self.__select)

        self.mProgressBar.setValue(0)
        self.mLabel.setText('')

    def __close(self):
        self.close()

    def __select(self):
        dlg = QFileDialog()
        dlg.setFileMode(QFileDialog.AnyFile)

        filenames = []
        if dlg.exec_():
            filenames = dlg.selectedFiles()

        if filenames:
            filename = filenames[0]
            self.mLineEdit.setText(filename)

    # def __populateProjComboBox(self):
    #     for deposit in config:
    #         self.mProjComboBox.addItem(deposit)
    #     self.mProjComboBox.setCurrentIndex(self.mProjComboBox.findText(self.deposit) if self.mProjComboBox.findText(self.deposit) != -1 else 0)
    #     return config

    def __export(self):
        filename = self.mLineEdit.text()
        if not filename:
            return

        deposit = self.deposit if self.deposit != None else 'DEFAULT'
        self.mSelectButton.setEnabled(False)
        self.mCancelButton.setEnabled(False)
        self.mImportButton.setEnabled(False)

        self.mLabel.setText(self.tr('Export database...'))
        self.mProgressBar.setMinimum(0)
        self.mProgressBar.setMaximum(0)
        qApp.processEvents()
        if deposit != 'DEFAULT':
            customProj=[float(self.config[deposit][i]) for i in ['x','y','z']]
            self.data.translate(customProj, inOut=False)
            print (customProj)
            self.data.export(self.mLineEdit.text())
            self.data.translate(customProj)
        else :
            self.data.export(self.mLineEdit.text())

        self.mLabel.setText(self.tr('Database correctly exported.'))
        self.mCancelButton.setEnabled(True)
        self.mProgressBar.setMaximum(100)
        self.mProgressBar.setValue(100)
        self.mCancelButton.setEnabled(True)
