import os

import serial.tools.list_ports

from PyQt5 import QtWidgets, uic
from PyQt5.QtCore import Qt, QSettings
from PyQt5.QtGui import QIntValidator

from .custom_projection_edition import CtrlCraftCustomProjEdition

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'ui', 'main_parameters.ui'))

settings = QSettings()


class CtrlCraftMainParametersDock(QtWidgets.QDockWidget, FORM_CLASS):

    def __init__(self, iface, data, app, parent=None):
        """Constructor."""
        super(CtrlCraftMainParametersDock, self).__init__(parent)
        self.iface = iface
        self.data = data
        self.app = app
        self.setupUi(self)
        self.mTruck1LineEdit.setValidator(QIntValidator(000,999))
        self.mTruck2LineEdit.setValidator(QIntValidator(000,999))
        self.mTruck3LineEdit.setValidator(QIntValidator(000,999))
        self.mTruck4LineEdit.setValidator(QIntValidator(000,999))

        if 'CTRLCRAFT_ADVANCEDUSER' in os.environ:
            if int(os.environ['CTRLCRAFT_ADVANCEDUSER']) == True:
                self.mGammaDownSpinBox.setEnabled(True)
                self.mGammaUpSpinBox.setEnabled(True)
                self.mProjButton.setEnabled(True)
        else :
            self.mGammaDownSpinBox.setEnabled(False)
            self.mGammaUpSpinBox.setEnabled(False)
            self.mProjButton.setEnabled(False)
        self.mCancelButton.clicked.connect(self.__close)
        self.mProjButton.clicked.connect(self.__editProj)
        self.mSaveButton.clicked.connect(self.__save)

        self.__update()
        self.iface.addDockWidget(Qt.LeftDockWidgetArea, self)
        self.isFloating()
        self.hide()


    def __close(self):
        self.close()

    def __populateGPSComboBox(self, current_value):
        gps_dict={0: 'Undetermined', 1: 'GNSS', 7: 'DGNSS', 15: 'Float', 23: 'Fixed'}
        for mode in gps_dict.values():
            self.mGPSComboBox.addItem(mode)
        self.mGPSComboBox.setCurrentIndex(self.mGPSComboBox.findText(current_value) if self.mGPSComboBox.findText(current_value) != -1 else 0)
        
    def __populateSerialComboBoxes(self):
        self.mSerialPortRover.clear()
        self.mSerialPortBase.clear()
        self.mSerialPortSpare.clear()
        comports = serial.tools.list_ports.comports()
        self.mSerialPortRover.addItem("")
        self.mSerialPortBase.addItem("")
        self.mSerialPortSpare.addItem("")
        for i, com in enumerate(comports):
            self.mSerialPortRover.addItem(com[0])
            self.mSerialPortBase.addItem(com[0])
            self.mSerialPortSpare.addItem(com[0])
            information = "Ports: {}\nDescription: {}\nHWID: {}".format(*com)
            self.mSerialPortRover.setItemData(i+1, information, Qt.ToolTipRole)
            self.mSerialPortBase.setItemData(i+1, information, Qt.ToolTipRole)
            self.mSerialPortSpare.setItemData(i+1, information, Qt.ToolTipRole)
        
        self.mSerialPortRover.setCurrentIndex(self.mSerialPortRover.findText(str(settings.value('/GeoCube/serialportrover'))) if self.mSerialPortRover.findText(str(settings.value('/GeoCube/serialportrover'))) != -1 else 0)
        self.mSerialPortBase.setCurrentIndex(self.mSerialPortBase.findText(str(settings.value('/GeoCube/serialportspare'))) if self.mSerialPortBase.findText(str(settings.value('/GeoCube/serialportbase'))) != -1 else 0)
        self.mSerialPortSpare.setCurrentIndex(self.mSerialPortSpare.findText(str(settings.value('/GeoCube/serialportspare'))) if self.mSerialPortSpare.findText(str(settings.value('/GeoCube/serialportspare'))) != -1 else 0)
            
    def __editProj(self):
        """ Open a edit windows for editing proj.ini file """
        proj_path = os.path.join(os.path.dirname(__file__),'..','proj.ini')
        edit = CtrlCraftCustomProjEdition(proj_path)
        self.mProjButton.setEnabled(False)
        self.mSaveButton.setEnabled(False)
        self.mCancelButton.setEnabled(False)
        edit.show()
        edit.exec_()
        self.mProjButton.setEnabled(True)
        self.mSaveButton.setEnabled(True)
        self.mCancelButton.setEnabled(True)

    def __validate(self):
        validator = QIntValidator(000,999)
        res=[]
        res.append([validator.validate(self.mTruck1LineEdit.text(),2)[0],len(self.mTruck1LineEdit.text())])
        res.append([validator.validate(self.mTruck2LineEdit.text(),2)[0],len(self.mTruck2LineEdit.text())])
        res.append([validator.validate(self.mTruck3LineEdit.text(),2)[0],len(self.mTruck3LineEdit.text())])
        res.append([validator.validate(self.mTruck4LineEdit.text(),2)[0],len(self.mTruck4LineEdit.text())])
        test = True
        for i in res:
            if i != [2,3]:
                test = False
        if test == False :
            QtWidgets.QMessageBox.warning(None, "Truck ID",
                self.tr("Truck ID must be a integer of 3 digits"))
        return test

    def __update(self):
        """ Init parameters in the different boxes """
        self.mTruck1LineEdit.setText(settings.value( '/CtrlCraft/Truck1/id'))
        self.mTruck1SpinBox.setValue(int(settings.value( '/CtrlCraft/Truck1/sc', 8)))
        self.mTruck2LineEdit.setText(settings.value( '/CtrlCraft/Truck2/id'))
        self.mTruck2SpinBox.setValue(int(settings.value( '/CtrlCraft/Truck2/sc', 8)))
        self.mTruck3LineEdit.setText(settings.value( '/CtrlCraft/Truck3/id'))
        self.mTruck3SpinBox.setValue(int(settings.value( '/CtrlCraft/Truck3/sc', 8)))
        self.mTruck4LineEdit.setText(settings.value( '/CtrlCraft/Truck4/id'))
        self.mTruck4SpinBox.setValue(int(settings.value( '/CtrlCraft/Truck4/sc', 8)))
        self.__populateGPSComboBox(str(settings.value( '/GeoCube/GPSmode')))
        self.mAngleSpinBox.setValue(float(settings.value( '/GeoCube/angle', 10)))
        self.mGammaDownSpinBox.setValue(float(settings.value( '/GeoCube/gammalow', 1000)))
        self.mGammaUpSpinBox.setValue(float(settings.value( '/GeoCube/gammaup', 1000)))
        self.__populateSerialComboBoxes()

    def __save(self):
        """ Save new parameters and actualize database"""
        test = self.__validate()
        if test == True :
            settings.setValue( '/CtrlCraft/Truck1/id',self.mTruck1LineEdit.text())
            settings.setValue( '/CtrlCraft/Truck1/sc',self.mTruck1SpinBox.value())
            settings.setValue( '/CtrlCraft/Truck2/id',self.mTruck2LineEdit.text())
            settings.setValue( '/CtrlCraft/Truck2/sc',self.mTruck2SpinBox.value())
            settings.setValue( '/CtrlCraft/Truck3/id',self.mTruck3LineEdit.text())
            settings.setValue( '/CtrlCraft/Truck3/sc',self.mTruck3SpinBox.value())
            settings.setValue( '/CtrlCraft/Truck4/id',self.mTruck4LineEdit.text())
            settings.setValue( '/CtrlCraft/Truck4/sc',self.mTruck4SpinBox.value())
            settings.setValue( '/GeoCube/GPSmode',self.mGPSComboBox.currentText())
            settings.setValue( '/GeoCube/angle',self.mAngleSpinBox.value())
            settings.setValue( '/GeoCube/gammalow',self.mGammaDownSpinBox.value())
            settings.setValue( '/GeoCube/gammaup',self.mGammaUpSpinBox.value())
            settings.setValue('/GeoCube/serialportrover', self.mSerialPortRover.currentText())
            settings.setValue('/GeoCube/serialportbase', self.mSerialPortBase.currentText())
            settings.setValue('/GeoCube/serialportspare', self.mSerialPortSpare.currentText())
            self.data.truck_ids_change([self.mTruck1LineEdit.text(),self.mTruck2LineEdit.text(),self.mTruck3LineEdit.text(),self.mTruck4LineEdit.text()])
            self.app.toolbar.close()
            self.app.toolbar_office.close()
            self.app.toolbar = None
            self.app.initToolBarMobile()
            self.app.toolbar_office = None
            self.app.initToolBarOffice()
            self.app.resetProject()
            self.close()
