# -*- coding: UTF-8 -*-

import numpy
from OpenGL.GL import *
from OpenGL.GL import shaders

from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import QVector3D

from .utility import computeNormals
import psycopg2
from shapely import wkb
from colour import Color

from .settings_data import GRADIENTS
from builtins import bytes

import time

class Scene(QObject):

    def __del__(self):
        self.__con.close()

    def __init__(self, conn_info, param, parent=None):
        super(Scene, self).__init__(parent)
        self.__zScale = 1.  # must be one here
        self.__old_param = {
                "slab": False
                }
        self.__param = param
        self.__bbox = {
                "xmin": 0,
                "ymin": 0,
                "xmax": 0,
                "ymax": 0
            }
        self.__cut = {
                "xmin": 0,
                "ymin": 0,
                "xmax": 0,
                "ymax": 0
            }
        self.conn_info = conn_info

        self.__prepareGradients()

        self.__con = psycopg2.connect(self.conn_info)
        self.__cur = self.__con.cursor()
        # TODO translate slabs on 0,0
        self.__cur.execute("""
            select st_3dextent(s.geom)
            from ctrlcraft.slabs_available s
            where s.panel = '{panel}' and s.blast = '{blast}'
            """.format(panel=self.__param['panel'], blast=self.__param['blast']))
        ext = self.__cur.fetchone()[0].replace('BOX3D(', '').replace(')', '').split(',')
        #self.__con.close()
        ext = [[float(c) for c in ext[0].split()], [float(c) for c in ext[1].split()]]

        self.__center = QVector3D(
                .5 * (ext[0][0] + ext[1][0]),
                .5 * (ext[0][1] + ext[1][1]),
                .5 * (ext[0][2] + ext[1][2]))
        self.center = QVector3D(0, 0, 0)
        self.extent = (ext[0][0] - self.__center.x(), ext[0][1] - self.__center.y(), ext[1][0] - self.__center.x(), ext[1][1] - self.__center.y())

        self.__extent = (ext[0][0] - self.__center.x(), ext[0][1] - self.__center.y(), ext[0][2] - self.__center.z(),
                ext[1][0] - self.__center.x(), ext[1][1] - self.__center.y(), ext[1][2] - self.__center.z())

        self.vtx = {
                "slab": None}
        self.idx = {
                "slab": None}
        self.nrml = {
                "slab": None}
        self.colors = {
                "slab": None}

        self.__labels = []

    def close(self):
        self.__con.close()

    def rendergl(self, leftv, upv, eye, height, context):
        glEnable(GL_DEPTH_TEST)
        glLightModelfv(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE)

        #glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE)

        glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,  [1., 1., 1., 1.])
        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,  [1., 1., 1., 1.])
        glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, [1., 1., 1., 1.])
        glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 50)

        glEnable(GL_COLOR_MATERIAL)

        glEnableClientState(GL_COLOR_ARRAY)
        glEnableClientState(GL_VERTEX_ARRAY)  # read vertex from buffer in graphic card
        glDisableClientState(GL_TEXTURE_COORD_ARRAY)  # no texture
        glEnableClientState(GL_NORMAL_ARRAY)  # create normale array

        for layer in ['slab']:
            if self.__param[layer]:
                if self.__param != self.__old_param:
                    self.__update(layer)
                if len(self.vtx[layer]):
                    glVertexPointerf(self.vtx[layer])
                    glNormalPointerf(self.nrml[layer])
                    glColorPointer(3, GL_FLOAT, 0, self.colors[layer])
                    glDrawElementsui(GL_QUADS, self.idx[layer])

        glDisableClientState(GL_NORMAL_ARRAY)
        glDisableClientState(GL_COLOR_ARRAY)
        glDisable(GL_COLOR_MATERIAL)

        # draw extend
        xm, ym, zm, xM, yM, zM = self.__extent
        glDisable(GL_LIGHTING)
        glDisable(GL_COLOR_MATERIAL)
        glColor4f(1., 0., 0., 1.)
        glBegin(GL_LINES)
        glVertex3f( xm, ym, zM)
        glVertex3f( xM, ym, zM)

        glVertex3f( xM, ym, zM)
        glVertex3f( xM, yM, zM)

        glVertex3f( xM, yM, zM)
        glVertex3f( xm, yM, zM)

        glVertex3f( xm, yM, zM)
        glVertex3f( xm, ym, zM)

        glVertex3f( xm, ym, zm)
        glVertex3f( xM, ym, zm)

        glVertex3f( xM, ym, zm)
        glVertex3f( xM, yM, zm)

        glVertex3f( xM, yM, zm)
        glVertex3f( xm, yM, zm)

        glVertex3f( xm, yM, zm)
        glVertex3f( xm, ym, zm)

        glVertex3f( xm, ym, zm)
        glVertex3f( xm, ym, zM)

        glVertex3f( xM, ym, zm)
        glVertex3f( xM, ym, zM)

        glVertex3f( xM, yM, zm)
        glVertex3f( xM, yM, zM)

        glVertex3f( xm, yM, zm)
        glVertex3f( xm, yM, zM)

        glEnd()

        glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION,  [0., 0., 0., 1.])

    def __update(self, layer):
        if layer == 'slab':
            #self.__con = psycopg2.connect(self.conn_info)
            #self.__cur = self.__con.cursor()
            # Get BBOX exten
            self.__getBBox()
            #self.__drawBBox() # TODO

            deltax = self.__bbox['xmax'] - self.__bbox['xmin']
            deltay = self.__bbox['ymax'] - self.__bbox['ymin']

            margin = 1
            self.__cut['xmax'] = (self.__param['xmax_cut'] * deltax / 100) + self.__bbox['xmin'] + margin
            self.__cut['ymax'] = (self.__param['ymax_cut'] * deltay / 100) + self.__bbox['ymin'] + margin
            self.__cut['xmin'] = (self.__param['xmin_cut'] * deltax / 100) + self.__bbox['xmin'] - margin
            self.__cut['ymin'] = (self.__param['ymin_cut'] * deltay / 100) + self.__bbox['ymin'] - margin

            cutSql = """
                     and st_xmin(ST_Envelope(geom)) <= {cutXmax} and st_xmin(ST_Envelope(geom)) >= {cutXmin}
                     and st_ymin(ST_Envelope(geom)) <= {cutYmax} and st_ymin(ST_Envelope(geom)) >= {cutYmin}
                     """.format(cutXmax=self.__cut['xmax'], cutYmax=self.__cut['ymax'], cutXmin=self.__cut['xmin'], cutYmin=self.__cut['ymin'])

            radSql = " and (radiometry is not null)"
            if self.__param['radiometrySelection'] != '':
                radSql = self.__param['radiometrySelection']

            # Get slabs
            self.__cur.execute("""
                select radiometry, geom
                from ctrlcraft.slabs_available
                where panel = '{panel}' and blast = '{blast}'
                {cutSql}
                {radSql}
                """.format(panel=self.__param['panel'], blast=self.__param['blast'], cutSql=cutSql, radSql=radSql))
            res = self.__cur.fetchall()
            #self.__con.close()
            geoms = []
            radio = []

            for rec in res:
                geoms.append(wkb.loads(bytes.fromhex(rec[1])))
                radio.append(rec[0])

            #for multipoly in geoms:
            #    assert len(list(multipoly)) == 6
            #    for quad in multipoly:
            #        assert len(quad.exterior.coords) == 5

            self.vtx[layer] = numpy.require(numpy.array([quad.exterior.coords[:-1] for poly in geoms for quad in poly]).reshape((-1, 3)), numpy.float32, 'C')
            self.vtx[layer] -= numpy.array([self.__center.x(), self.__center.y(), self.__center.z()])

            if len(self.vtx[layer]):
                self.vtx[layer][:, 2] *= self.__zScale

            self.idx[layer] = numpy.require(numpy.arange(len(self.vtx[layer])).reshape((-1, 4)), numpy.int32, 'C')
            self.nrml[layer] = computeNormals(self.vtx[layer], self.idx[layer])

            # Prepare the RGB array by duplicate the radiometry value
            nbColorRgb, nbVtxPerFace, nbFacePerVolumne = 3, 4, 6
            self.colors[layer] = numpy.repeat(radio, nbColorRgb*nbVtxPerFace*nbFacePerVolumne).reshape((-1, 3))
            for r in self.colors[layer]:
                # Testing the radiometry value and replace by the corresponding RVB values
                r = self.__setColor(r)

        self.__old_param = self.__param

    def __getBBox(self):
        """ Get BBox of data """
        self.__cur.execute("""
            WITH env as (
            select ST_Envelope(st_collect(geom)) as geom
                            from ctrlcraft.slabs_available
                            where panel = '{panel}' and blast = '{blast}'
            )
            select st_xmin(geom) as xmin, st_ymin(geom) as ymin, st_xmax(geom) as xmax, st_ymax(geom) as ymax
            FROM env
            """.format(panel=self.__param['panel'], blast=self.__param['blast']))
        rec = self.__cur.fetchall()

        self.__bbox['xmin'] = rec[0][0]
        self.__bbox['ymin'] = rec[0][1]
        self.__bbox['xmax'] = rec[0][2]
        self.__bbox['ymax'] = rec[0][3]

    def __setColor(self, r):
        """ Affect colors to slab sides (r) according to radiometry value """

        ratio = 1
        rad = r[0]
        for g in GRADIENTS:
            if g["min"] == -1 and rad <= g["max"]:
                gradient = g["gradient"]
                break
            if g["max"] == -1 and rad >= g["min"]:
                gradient = g["gradient"]
                break
            if rad > g["min"] and rad <= g["max"]:
                ratio = (rad - g["min"]) / (g["max"] - g["min"])
                gradient = g["gradient"]
                break

        color = gradient[int(ratio * 100) - 1]

        r[0] = color.rgb[0]
        r[1] = color.rgb[1]
        r[2] = color.rgb[2]

        return r

    def __prepareGradients(self):
        """ Build gradient list for colors """
        gradients = []
        for r in GRADIENTS:
            colorStart = Color(rgb=(float(r["colStart"][0]) / 255.0, float(r["colStart"][1]) / 255.0, float(r["colStart"][2]) / 255.0))
            colorEnd = Color(rgb=(float(r["colEnd"][0]) / 255.0, float(r["colEnd"][1]) / 255.0, float(r["colEnd"][2]) / 255.0))
            r["gradient"] = list(colorStart.range_to(colorEnd, 100))

        return gradients

    def setZscale(self, scale):
        factor = float(scale) / self.__zScale

        for layer in ['slab']:
            if self.vtx[layer] is not None:
                self.vtx[layer][:, 2] *= factor
                if layer in ['slab']:
                    self.nrml[layer] = computeNormals(self.vtx[layer], self.idx[layer])

        for scatter in self.__labels:
            scatter['point'][2] *= factor

        self.__zScale = scale
